#:
MIME_URLENCODE = 'application/x-www-form-urlencoded'
#:
MIME_FORMDATA = 'multipart/form-data'
#:
MIME_PLAINTEXT = 'text/plain'

MIME_EPUB = 'application/epub+zip'
#:
MIME_JAR = 'application/java-archive'
#:
MIME_JS = 'application/jsavascript'
#:
MIME_JSON = 'application/json'
#:
MIME_MSWORD = 'application/msword'
#:
MIME_BIN = 'application/octet-stream'
#:
MIME_OGG = 'application/ogg'
#:
MIME_PDF = 'application/pdf'
#:
MIME_RTF = 'application/rtf'
#:
MIME_AZW = 'application/vnd.amazon.ebook'
#:
MIME_MPKG = 'application/vnd.apple.installer+xml'
#:
MIME_XUL = 'application/vnd.moliza.xul+xml'
#:
MIME_XLS = 'application/vnd.ms-excel'
#:
MIME_PPT = 'application/vnd.ms-powerpoint'
#:
MIME_ODP = 'application/vnd.oasis.opendocument.presentaion'
#:
MIME_ODS = 'application/vnd.oasis.opendocument.spreadsheet'
#:
MIME_ODT = 'application/vnd.oasis.opendocument.text'
#:
MIME_VSD = 'application/vnd.visio'
#:
MIME_7Z = 'application/x-7z-compressed'
#:
MIME_ABW = 'application/x-abiword'
#:
MIME_BZ = 'application/x-bzip'
#:
MIME_BZ2 = 'application/x-bzip2'
#:
MIME_CSH = 'application/x-csh'
#:
MIME_RAR = 'application/x-rar-compressed'
#:
MIME_SH = 'application/x-sh'
#:
MIME_SWF = 'application/x-shockwave-flash'
#:
MIME_TAR = 'application/x-tar'
#:
MIME_XHTML = 'application/xhtml+xml'
#:
MIME_XML = 'application/xml'
#:
MIME_ZIP = 'application/zip'
#:
MIME_AAC = 'audio/acc'
#:
MIME_MIDI = 'audio/midi'
#:
MIME_OGA = 'audio/ogg'
#:
MIME_WEBA = 'audio/webm'
#:
MIME_WAV = 'audio/x-wav'
#:
MIME_TTF = 'font/ttf'
#:
MIME_WOFF = 'font/woff'
#:
MIME_WOFF2 = 'font/woff2'
#:
MIME_GIF = 'image/gif'
#:
MIME_JPEG = 'image/jpeg'
#:
MIME_SVG = 'image/svg+xml'
#:
MIME_TIFF = 'image/tiff'
#:
MIME_WEBP = 'image/webp'
#:
MIME_ICO = 'image/x-icon'
#:
MIME_ICS = 'text/calendar'
#:
MIME_CSS = 'text/css'
#:
MIME_CSV = 'text/csv'
#:
MIME_HTML = 'text/html'
#:
MIME_3GP = 'video/3gpp'
#:
MIME_3G2 = 'video/3gpp2'
#:
MIME_MPEG = 'video/mpeg'
#:
MIME_OGV = 'video/ogg'
#:
MIME_WEBM = 'video/webm'
#:
MIME_AVI = 'video/x-msvideo'
